/***********************************************************************************************

   LICENSE
   Copyright (C) 2018 openQCM
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see http://www.gnu.org/licenses/gpl-3.0.txt
  --------------------------------------------------------------------------------
   OPENQCM NEXT - Quartz Crystal Microbalance with dissipation monitoring, 
   multiovertone monitoring and temperature control
   openQCM is the unique opensource quartz crystal microbalance
   http://openqcm.com/

   ELECTRONICS
     - board and firmware designed for TEENSY 4.0 dev board
     - DDS/DAC Synthesizer AD9851
     - phase comparator AD8302
     - I2C digital potentiometer AD5251+
    
     - Transistor to turn on the TEC THORLAB MTD415T: PIN 10 = HIGH
     - Teensy pin control to enable/disable TEC THORLAB MTD415T: PIN 11 = LOW
     - MCP9808 temperature sensor for ambient temperature monitoring

   HISTORY CHANGES
   version 0.1.1c 
   MAJOR CHANGES
    - include ADC.h library in src directory 
    https://pedvide.github.io/ADC/ and https://github.com/pedvide/ADC
   
   version 0.1.1a 
   MAJOR CHNAGES 
    - Change the RGB pin definition and RGB colour function, accoring to electronic configuration 
    - Change the colour of onboard RGB led pin for different machine status
    - Change the ADC conversion speed to VERY_HIGH_SPEED to eliminate some noise in amplitude signal 
    - Change the temperature measurement via THORLAB MTD415T, even if temperatre control is turned off
    - Change the temperature control via THORLAB MTD415T ENABLE PIN 

   TEST MTD415T using enable control pin
   --------------------------------------------------------------------------------
   PROGRAMMING
   COMMAND   RESPONSE
   m?        Reads the version of hardware and software
   TEMPERATURE
   --------------------------------------------------------------------------------
   programming
   Tx        Sets the set temperature to x (x: 5000 to 45000 [10-3 °C])
   reading
   T?        Reads the set temperature (Value range x: 5000 to 45000 [10-3 °C])
   Te?       Reads the actual temperature
   PID Settings
   ----------------------------------------------------------------------------------------------------
   programming
   Cx        Sets the cycling time to x (Value range x: 1 to 1000 [msec])     default 50 msec
   Px        Sets the P Share to x  (Value range x: 0 to 100000 [mA/K])       default 1000 mA/K
   Ix        Sets the I Share to x  (Value range x: 0 to 100000 [mA/(K+sec)]) default 200 mA/(K*sec)
   Dx        Sets the D Share to x  (Value range x: 0 to 100000 [(mA*s)/K])   default 100 (mA*sec)/K
   ----------------------------------------------------------------------------------------------------

   TEST FREQUENCY INPUT STRING
   10000000.0;10050000.0;100

   --------------------------------------------------------------------------------
   version  0.1.1c
   date     2021-04-19
   author   marco - openQCM team
   --------------------------------------------------------------------------------

 ***********************************************************************************************/

/************************** LIBRARIES **************************/
#include <Wire.h>
#include "src/Adafruit_MCP9808.h"
// #include "Adafruit_MCP9808.h"
// #include "src/InternalTemperature.h"

// #include <ADC.h>
// #include <ADC_util.h>
# include "src/ADC/ADC.h"
# include "src/ADC/ADC_util.h"

/*************************** DEFINE ***************************/
// potentiometer AD5252 I2C address is 0x2C(44)
#define ADDRESS 0x2C
// potentiometer AD5252 default value for compatibility with openQCM Q-1 shield @5VDC
#define POT_VALUE 240 //254
// reference clock
#define REFCLK 125000000

#define AVERAGING   1
#define RESOLUTION 12

/*************************** VARIABLE DECLARATION ***************************/

// RGB LED
/* ***************************************************************************
int RGB_RED_PIN   = 7;
int RGB_GREEN_PIN = 6;
int RGB_BLUE_PIN  = 5;
*************************************************************************** */
int RGB_RED_PIN   = 5;
int RGB_GREEN_PIN = 6;
int RGB_BLUE_PIN  = 7;


// current input frequency
long freq = 0;
// DDS Synthesizer AD9851 pin function for arduino h7
int WCLK = 3;
int DATA = 4;
int FQ_UD = 2;
// frequency tuning word
long FTW;
float temp_FTW; // temporary variable
// phase comparator AD8302 pinout
//int AD8302_PHASE = 20;
//int AD8302_MAG = 37;
////int AD83202_REF = 17;
//int AD83202_REF = 34;

/*
  // Arduino H7 ADC
  int GAIN = A0;
  int PHASE = A1;
*/

// T40 pin ADC
const int readPin = A9;
const int readPin2 = A3;
ADC *adc = new ADC(); // adc object


// TODO
double val = 0;

// TODO delete
// Create the MCP9808 temperature sensor object
// Adafruit_MCP9808 tempsensor = Adafruit_MCP9808();

// init temperature variable
// float temperature = 0;

// LED pin
//int ledPin1 = 24;
//int ledPin2 = 25;

// ADC init variabl
boolean WAIT = false;
// ADC waiting delay microseconds
int WAIT_DELAY_US = 100;
// ADC averaging
boolean AVERAGING_BOOL = true;
// inint number of averaging
int AVERAGE_SAMPLE = 2048;
// teensy ADC averaging init
// int ADC_RESOLUTION = 13;

// init sweep param
long freq_start;
long freq_stop;
long freq_step;

// init output ad8302 measurement (cast to double)
double measure_phase = 0;
double measure_mag = 0;

/* MTD415T variable delcaration */
// MTD415T enable input connected to Arduino H7 digital pin 5
//int ENABLE_PIN = 6;
//int STATUS_PIN = 7;

// TODO disabled ENABLE_PIN
// int ENABLE_PIN = 10;

// Status Signal Output
// HIGH = Temperature within defined temperature window
// LOW  = Temperature outside programmed temperature window or an error occurred
int STATUS_TEC = 9;

// enabled HIGH, didabeld LOW
// TODO CANCEL
// int CTRL_SWITCH = 8;

/* -------------------------------------------------
// 3-pin control for enable - disable TEC controller
int CTRL_SWITCH_1 = 10;
int CTRL_SWITCH_2 = 11;
int CTRL_SWITCH_3 = 12;
 --------------------------------------------------*/

// Transistor to turn on the TEC THORLAB MTD415T: PIN 10 = HIGH
int CTRL_SWITCH_PIN = 10; 
// Teensy pin control to enable/disable TEC THORLAB MTD415T: PIN 11 = LOW
int ENABLE_PIN = 11; 
// set boolean temperature control switch OFF
boolean CTRL_SWITCH = false;

// TODO embedded light on T40 delete ?
const int ledPin = 13;

/*
// T40 internal temperature variable init
float temperature_internal = 0.0;
// temperature difference between thermistor and internal temperature
float temperature_difference = 0.0;
*/
// Create the MCP9808 temperature sensor object
Adafruit_MCP9808 tempsensor = Adafruit_MCP9808();
// init temperature variable
float temperature = 0;

/*************************** FUNCTION ***************************/

/* AD9851 set frequency fucntion */
void SetFreq(long frequency)
{
  // set to 125 MHz internal clock
  temp_FTW = (frequency * pow(2, 32)) / REFCLK;
  FTW = long (temp_FTW);

  long pointer = 1;
  int pointer2 = 0b10000000;
  int lastByte = 0b10000000;

  /* 32 bit dds tuning word frequency instructions */
  for (int i = 0; i < 32; i++)
  {
    if ((FTW & pointer) > 0) digitalWrite(DATA, HIGH);
    else digitalWrite(DATA, LOW);
    digitalWrite(WCLK, HIGH);
    digitalWrite(WCLK, LOW);
    pointer = pointer << 1;
  }

  /* 8 bit dds phase and x6 multiplier refclock*/
  for (int i = 0; i < 8; i++)
  {
    //if ((lastByte & pointer2) > 0) digitalWrite(DATA, HIGH);
    //else digitalWrite(DATA, LOW);
    digitalWrite(DATA, LOW);
    digitalWrite(WCLK, HIGH);
    digitalWrite(WCLK, LOW);
    pointer2 = pointer2 >> 1;
  }

  digitalWrite(FQ_UD, HIGH);
  digitalWrite(FQ_UD, LOW);

  //FTW = 0;
}

// TODO DELETE BLINK
void my_blink() {
  digitalWrite(ledPin, HIGH);   // set the LED on
  delay(100);                  // wait for a second
  digitalWrite(ledPin, LOW);    // set the LED off
  delay(100);
}

// RGB LED function
// RGB light value 0,..., 255
void RGB_color(int red_light_value, int green_light_value, int blue_light_value)
{
  analogWrite(RGB_RED_PIN, 255 - red_light_value);
  analogWrite(RGB_GREEN_PIN, 255 - green_light_value);
  analogWrite(RGB_BLUE_PIN, 255 - blue_light_value);
}


/*************************** SETUP ***************************/
void setup()
{
  // Initialise I2C communication as Master
  Wire.begin();
  // Initialise serial communication, set baud rate = 9600
  Serial.begin(115200);
  // DEBUG_0.1.1a
  Serial.setTimeout(250); 

  // set potentiometer value
  // Start I2C transmission
  Wire.beginTransmission(ADDRESS);
  // Send instruction for POT channel-0
  Wire.write(0x01);
  // Input resistance value, 0x80(128)
  Wire.write(POT_VALUE);
  // Stop I2C transmission
  Wire.endTransmission();

  // AD9851 set pin mode
  pinMode(WCLK, OUTPUT);
  pinMode(DATA, OUTPUT);
  pinMode(FQ_UD, OUTPUT);

  // AD9851 enter serial mode
  digitalWrite(WCLK, HIGH);
  digitalWrite(WCLK, LOW);
  digitalWrite(FQ_UD, HIGH);
  digitalWrite(FQ_UD, LOW);

  // AD8302 set pin mode
  //  pinMode(AD8302_PHASE, INPUT);
  //  pinMode(AD8302_MAG, INPUT);
  //  pinMode(AD83202_REF, INPUT);

  /*
    // set adc mode
    pinMode(GAIN, INPUT);
    pinMode(PHASE, INPUT);
    // set analog read resolution
    analogReadResolution(16);
  */

  // T40 ADC SETTING

  pinMode(readPin, INPUT);
  pinMode(readPin2, INPUT);

  adc->setAveraging(AVERAGING); // set number of averages
  adc->setResolution(RESOLUTION); // set bits of resolution

  // it can be any of the ADC_CONVERSION_SPEED enum: VERY_LOW_SPEED, LOW_SPEED, MED_SPEED, HIGH_SPEED_16BITS, HIGH_SPEED or VERY_HIGH_SPEED
  // see the documentation for more information
  // additionally the conversion speed can also be ADACK_2_4, ADACK_4_0, ADACK_5_2 and ADACK_6_2,
  // where the numbers are the frequency of the ADC clock in MHz and are independent on the bus speed.
  adc->setConversionSpeed(ADC_CONVERSION_SPEED::VERY_HIGH_SPEED); // change the conversion speed
  // it can be any of the ADC_MED_SPEED enum: VERY_LOW_SPEED, LOW_SPEED, MED_SPEED, HIGH_SPEED or VERY_HIGH_SPEED
  adc->setSamplingSpeed(ADC_SAMPLING_SPEED::VERY_HIGH_SPEED); // change the sampling speed

  ////// ADC1 /////
  adc->setAveraging(AVERAGING, ADC_1); // set number of averages
  adc->setResolution(RESOLUTION, ADC_1); // set bits of resolution
  adc->setConversionSpeed(ADC_CONVERSION_SPEED::VERY_HIGH_SPEED, ADC_1); // change the conversion speed
  adc->setSamplingSpeed(ADC_SAMPLING_SPEED::VERY_HIGH_SPEED, ADC_1); // change the sampling speed

  adc->startSynchronizedContinuous(readPin, readPin2);


  //  // Teensy 3.6 set  adc resolution
  //  analogReadResolution(ADC_RESOLUTION);

  // begin MCP9808 sensor temperature sensor
  tempsensor.begin();

  // turn on the light
  //  pinMode(ledPin1, OUTPUT);
  //  pinMode(ledPin2, OUTPUT);
  //  digitalWrite(ledPin1, HIGH);
  //  digitalWrite(ledPin2, HIGH);

  // SetFreq(1000000);

  // analogReference(AR_DEFAULT);

  /* MTD415T SETUP */
  // begin T40 UART Serial1
  Serial1.begin(115200);
  // DEBUG_0.1.1a
  Serial1.setTimeout(250);


  // TODO delete call to enable pin
  // set enabled pin mode
  // pinMode(ENABLE_PIN, OUTPUT);
  // MTD415T enable input Low = enabled, High = Disabled
  // TODO delete ENABLE_PIN
  // digitalWrite(ENABLE_PIN, LOW);

  // set status signal pin
  pinMode(STATUS_TEC, INPUT);

  // TODO cancel
  // PELTIER control switch enabled LOW
  // pinMode (CTRL_SWITCH, OUTPUT);
  // enable
  // digitalWrite(CTRL_SWITCH, LOW);

  /* ----------------------------
  pinMode(CTRL_SWITCH_1, OUTPUT);
  pinMode(CTRL_SWITCH_2, OUTPUT);
  pinMode(CTRL_SWITCH_3, OUTPUT);
  ---------------------------- */
  // enable
  /* ----------------------------
  digitalWrite(CTRL_SWITCH_1, HIGH);
  digitalWrite(CTRL_SWITCH_2, HIGH);
  digitalWrite(CTRL_SWITCH_3, HIGH);
  ---------------------------- */
  /*
    // disable at startup
    digitalWrite(CTRL_SWITCH_1, LOW);
    digitalWrite(CTRL_SWITCH_2, LOW);
    digitalWrite(CTRL_SWITCH_3, LOW);
  */

  /* TEC INIT and SETTING */
  /////////////////////////////////////
  // init control switch and enable pin 
  pinMode(CTRL_SWITCH_PIN, OUTPUT); 
  pinMode(ENABLE_PIN, OUTPUT); 
  // turn on TEC default
  digitalWrite(CTRL_SWITCH_PIN, HIGH);
  // disable the TEC default (LOW active)
  digitalWrite(ENABLE_PIN, HIGH);
  /////////////////////////////////////

  // TODO delete
  pinMode(ledPin, OUTPUT);

  // RGB LED SETUP
  pinMode(RGB_RED_PIN, OUTPUT);
  pinMode(RGB_GREEN_PIN, OUTPUT);
  pinMode(RGB_BLUE_PIN, OUTPUT);
  
  // shine on you crazy diamond, openqcm (0, 142, 192)
  RGB_color(0, 142, 192);

  /* ----------------------------------------------------------------
  // CALIBRATE TEMPERATURE INTERNAL TODO REMOVE

  // measure T40 internal temperature
  temperature_internal = InternalTemperature.readTemperatureC();
  // Serial.println("Temperature internal = ");
  // Serial.println(temperature_internal);

  // flush initial buffer
  for  (int nn = 0; nn < 10; nn++) {
    Serial1.readStringUntil('\n');
    delay(20);
  }

  // measure thermistor temperature
  Serial1.println("Te?");
  String temperature_now_string = Serial1.readStringUntil('\n');

  float temperature_now = temperature_now_string.toFloat() / 1000.0;
  // Serial.println("Temperature thermistor = ");
  // Serial.println(temperature_now);

  // TODO then turn off again TEC code:..........
  delay(100);
  // calculate temperature difference for calibration (dummuy)
  temperature_difference = temperature_internal - temperature_now;
  // Serial.println(temperature_difference );
  ---------------------------------------------------------------- */

  delay(100);
  
  // disable tec
  // disable at startup
//  digitalWrite(CTRL_SWITCH_1, LOW);
//  digitalWrite(CTRL_SWITCH_2, LOW);
//  digitalWrite(CTRL_SWITCH_3, LOW);

}

int message = 0;
// boolean debug = true;
long pre_time = 0;
long last_time = 0;

int byteAtPort = 0;

/* MTD415T string test init */
// init first setting if temperature module
boolean TEMPERATURE_BOOLEAN = true;
// init read string
String MTD415T_READ_STRING = "";
// PROGRAM
String TEMPERATURE_SET = "T20000\n";
String TEMPERATURE_GET = "Te?\n";
String P_SET = "P1000\n";
String P_GET = "";

String TEMPERATURE_SET_READ = "T?";
String TEMPERATURE_READ = "Te?";

String message_str = "";
String readStr = "";

// T40 init ADC
double value = 0;
double value2 = 0;
long time_start = 0;

ADC::Sync_result result;

unsigned long MAIN_COUNTER = 0;

// TODO delete temporary debug var
boolean DBG_TEMP = false;
String my_string = "";
String read_message_TEMP = "";

/*************************** LOOP ***************************/
void loop()
{
  // INIT PELTIER MODULE
  // TODO check the default parameters for the init
  if (TEMPERATURE_BOOLEAN) {
    //delay(1000);
    // setup the MTD415T just once in my life
    // Serial1.print(P_SET);
    Serial1.println(P_SET);
    if (Serial1.available()) {
      // read message from MTD415T
      // Serial.write(Serial1.read());

      // MTD415T_READ_STRING = Serial1.readStringUntil('\n');
    }
    Serial.print(MTD415T_READ_STRING);
    //delay(250);

    // Serial1.print(TEMPERATURE_SET);
    Serial1.println(TEMPERATURE_SET);

    if (Serial1.available()) {
      // read message from MTD415T
      // Serial.write(Serial1.read());

      // MTD415T_READ_STRING = Serial1.readStringUntil('\n');
    }
    Serial.print(MTD415T_READ_STRING);
    //delay(250);
    TEMPERATURE_BOOLEAN = false;
  }

  // check byte at serialport
  if ( (byteAtPort = Serial.available()) > 0 ) {

    // TODO
    // reset the case switch variable
    message = 0;
    
    if (DBG_TEMP) {
      MAIN_COUNTER++;
      Serial.print("MAIN COUNTER = ");
      Serial.println(MAIN_COUNTER);
    }

    // read string message at serial port
    message_str = Serial.readStringUntil('\n');
    // convert string to byte artay
    char buf[byteAtPort];
    message_str.toCharArray(buf, sizeof(buf));

    // DECODE MESSAGE at SERIAL PORT check first byte
    // TEMPERATURE SETTING
    if (buf[0] == 'T') {
      // send message to Peltier Module
      // Serial1.print(message_str);
      Serial1.println(message_str);

      // TODO switch case here please
      if (message_str == TEMPERATURE_SET_READ) {
        Serial.println("reading the set temperature");
        // Serial.println(MTD415T_READ_STRING);
        Serial.println(Serial1.readStringUntil('\n'));
      }
      else if (message_str == TEMPERATURE_READ) {
        Serial.println("reading the actual temperature");
        // Serial.println(MTD415T_READ_STRING);
        Serial.println(Serial1.readStringUntil('\n'));
      }
      else {
        Serial1.readStringUntil('\n');
      }

      /* TODO DELETE HERE THE READ MESSAGE
        // read message from Peltier module
        if (Serial1.available()) {
        // TODO delete
        my_string = Serial1.readStringUntil('\n');
        // DEBUG
        if (DBG){
          Serial.println("message read from Peltier if temperature is set");
          Serial.println(my_string);
        }
        }
      */

      // TODO inutility delete my_blink() replace with RCB led message
      // my_blink();

    }
    // PID SETTING
    // cycling time
    else if ( buf[0] == 'C' ) {
      // send message to Peltier Module
      Serial1.println(message_str);
      // read message
      read_message_TEMP = Serial1.readStringUntil('\n');
      if (DBG_TEMP) Serial.println(read_message_TEMP );
      if ( buf[1] == '?') Serial.println(read_message_TEMP );
    }
    // P Share
    else if ( buf[0] == 'P' ) {
      // send message to Peltier Module
      Serial1.println(message_str);
      // read message
      read_message_TEMP = Serial1.readStringUntil('\n');
      if (DBG_TEMP) Serial.println(read_message_TEMP );
      if ( buf[1] == '?') Serial.println(read_message_TEMP );
    }
    // I Share
    else if ( buf[0] == 'I' ) {
      // send message to Peltier Module
      Serial1.println(message_str);
      // read message
      read_message_TEMP = Serial1.readStringUntil('\n');
      if (DBG_TEMP) Serial.println(read_message_TEMP );
      if ( buf[1] == '?') Serial.println(read_message_TEMP );
    }
    // D Share
    else if ( buf[0]  == 'D' ) {
      // send message to Peltier Module
      Serial1.println(message_str);
      // read message
      read_message_TEMP = Serial1.readStringUntil('\n');
      if (DBG_TEMP) Serial.println(read_message_TEMP );
      if ( buf[1] == '?') Serial.println(read_message_TEMP );
    }

    else if (buf[0]  == 'X' ) {
      // Serial.println("TURN THE TEC ...");
      if ( buf[1] == '1') {
//        
//        // Serial.println("ON");
//        digitalWrite(CTRL_SWITCH_1, HIGH);
//        digitalWrite(CTRL_SWITCH_2, HIGH);
//        digitalWrite(CTRL_SWITCH_3, HIGH);

        // turn ON  TEC 
        // digitalWrite(CTRL_SWITCH_PIN, HIGH);
        
        // ENABLE  the TEC default (LOW active)
        digitalWrite(ENABLE_PIN, LOW);
        // set boolean control true
        CTRL_SWITCH = true;
        // colour green ???
        // RGB_color(0, 254, 0);
        RGB_color(0, 142, 192);
      }

      if ( buf[1] == '0') {
        
//        // Serial.println("OFF");
//        digitalWrite(CTRL_SWITCH_1, LOW);
//        digitalWrite(CTRL_SWITCH_2, LOW);
//        digitalWrite(CTRL_SWITCH_3, LOW);

        // turn OFF TEC 
        // digitalWrite(CTRL_SWITCH_PIN, LOW);
        
        // DISABLE the TEC default (LOW active)
        digitalWrite(ENABLE_PIN, HIGH); 
        // // set boolean control false
        CTRL_SWITCH = false;
        // colour yellow
        RGB_color(255, 255, 0);
      }
    }

    // GET SWEEP FREQUENCY PARAMETERS
    else {
      char *p = buf;
      char *str;
      int nn = 0;
      // decode message
      while ((str = strtok_r(p, ";", &p)) != NULL) { // delimiter is the semicolon
        // frequency start
        if (nn == 0) {
          freq_start = atol(str);
          nn = 1;
        }
        // frequency stop
        else if (nn == 1) {
          freq_stop = atol(str);
          nn = 2;
        }
        // frequency step
        else if (nn == 2) {
          freq_step = atol(str);
          nn = 0;
          message = 1;
        }
      }
    }

    // DUMMY DO NOTHING
    if (message == 0) {
    }

    // START FREQUENCY SWEEP
    if (message == 1) {
      // start sweep
      long count = 0;
      pre_time = millis();
      // start sweep cycle measurement
      for (count = freq_start; count <= freq_stop; count = count + freq_step)
      {
        // set AD9851 DDS current frequency
        SetFreq(count);
        // do the magic ! waiting for the ADC measure
        // if (WAIT) delayMicroseconds(WAIT_DELAY_US);

        // measure gain phase
        //        int  app_phase = 0;
        //        int app_mag = 0;

        // ADC measure and averaging
        if (AVERAGING_BOOL == true) {
          for (int i = 0; i < AVERAGE_SAMPLE; i++) {
            result = adc->readSynchronizedContinuous();
            value += (uint16_t)result.result_adc0;
            value2 += (uint16_t)result.result_adc1;
          }
          // averaging (cast to double)
          value2 = 1.0 * value2 / AVERAGE_SAMPLE;
          value = 1.0 * value / AVERAGE_SAMPLE;

          // serial write data
          Serial.print(value);
          Serial.print(";");
          Serial.print(value2);
          Serial.println();
        }
      }
      /************************************************************************************
        // measure temperature
        tempsensor.shutdown_wake(0);  // Don't remove this line! required before reading temp
        temperature = tempsensor.readTempC();
        // serial write temperature data at the end of the sweep
      ************************************************************************************/
      // send message = read tempearature to Peltier Module
      // Serial1.print(TEMPERATURE_GET);
      Serial1.println(TEMPERATURE_GET);

      /* DELETE THIS IN SERIAL AVAILABLE LOOP
        // get value of CURRENT temperature value (?)
        if (Serial1.available()) {
        // read message from MTD415T
        // Serial.write(Serial1.read());
        MTD415T_READ_STRING = Serial1.readStringUntil('\n');
        }
        else{
        MTD415T_READ_STRING = "";
        }
      */

      // print temperature data to serial port
      if (CTRL_SWITCH == false){
        /* -----------------------------------------------------------  
        // measure internal temperature 
        temperature_internal = InternalTemperature.readTemperatureC();
        // calibrate 
        Serial.print(temperature_internal - temperature_difference);
        ----------------------------------------------------------- */
        /* ----------------------------------------------------------- 
        // measure temperature using MCP9808
        tempsensor.shutdown_wake(0);  // Don't remove this line! required before reading temp
        temperature = tempsensor.readTempC();
        // serial write temperature data at the end of the sweep
        Serial.print(temperature);
        ----------------------------------------------------------- */
        // measure thermistor temperature 
        Serial.print(MTD415T_READ_STRING.toFloat() / 1000.0);
      }
      else if (CTRL_SWITCH == true){
        // measure thermistor temperature 
        Serial.print(MTD415T_READ_STRING.toFloat() / 1000.0);
      }
      // Serial.print(temperature);
      Serial.print(";");
      // print termination char EOM
      Serial.print("s");
      //      Serial.println(millis()-pre_time);
      //      Serial.println();

      // TODO
      // reset the case switch variable
      // message = 0;
    }
  }
  
  // get current temperature value 
  if (Serial1.available()) {
    // read message from MTD415T
    // Serial.write(Serial1.read());
    MTD415T_READ_STRING = Serial1.readStringUntil('\n');
  }
  //  else{
  //        MTD415T_READ_STRING = "";
  //  }

  // check the status tec pin only if the PWR TEC is on
  if (CTRL_SWITCH == true && message == 1) {
    if (digitalRead(STATUS_TEC) == 1){
      // Status Signal (High = temperature within defined temperature window
      RGB_color(0, 142, 192);
    }
    // You don't have to put on the red light 
    // Low = Temperature outside programmed temperature window or an ERROR occurred
    else RGB_color(255, 0, 0);

//    // READ MTD415T status pin
//    if (digitalRead(STATUS_TEC) == 0) {
//      // RGB_color(254, 0, 0);
//      RGB_color(0, 255, 0);
//    }
//    else {
//      // get red light 
//      RGB_color(255, 0, 0);
//    }
  }

}
